#! python3  # noqa: E265

"""
    Widget to locate cadastre feature
"""

# standard

# PyQGIS
import os
from typing import List

from qgis.core import QgsGeometry, QgsLocatorFilter, QgsProject
from qgis.gui import QgisInterface, QgsLocatorWidget
from qgis.PyQt import uic
from qgis.PyQt.QtWidgets import QWidget

from locator_grand_lyon.core.adress_locator_filter import AdressLocatorFilter
from locator_grand_lyon.core.road_intersection_locator_filter import (
    RoadIntersectionLocatorFilter,
)
from locator_grand_lyon.core.road_locator_filter import RoadLocatorFilter
from locator_grand_lyon.core.vector_layer_locator_filter import VectorLayerLocatorFilter
from locator_grand_lyon.toolbelt.log_handler import PlgLogger

# project

# ############################################################################
# ########## Globals ###############
# ##################################


# ############################################################################
# ########## Classes ###############
# ##################################
class GrandLyonLocatorWidget(QWidget):
    """Widget to locate cadastre feature"""

    def __init__(self, parent: QWidget = None, iface: QgisInterface = None) -> None:
        super().__init__(parent)
        uic.loadUi(
            os.path.dirname(os.path.realpath(__file__)) + "/wdg_locator_filter.ui",
            self,
        )
        self.log = PlgLogger().log

        self.iface = iface

        self.wdg_locator.setMapCanvas(self.iface.mapCanvas())
        self.road_locator_filter = RoadLocatorFilter(self.iface)
        self.road_intersection_locator_filter = RoadIntersectionLocatorFilter(
            self.iface
        )
        self.adress_locator_filter = AdressLocatorFilter(self.iface)
        self.wdg_locator.locator().registerFilter(self.road_locator_filter)
        self.wdg_locator.locator().registerFilter(self.adress_locator_filter)

        self._init_all_locator_widget()

        self.wdg_locator_intersection.setMapCanvas(self.iface.mapCanvas())
        self.wdg_locator_intersection.setEnabled(False)
        self.wdg_locator_intersection.locator().registerFilter(
            self.road_intersection_locator_filter
        )

        self.adress_locator_filter.resultSelected.connect(self.adress_selected)
        self.road_locator_filter.resultSelected.connect(self.road_selected)

        QgsProject.instance().layersAdded.connect(self._project_layer_updated)
        QgsProject.instance().layersRemoved.connect(self._project_layer_updated)

    def _init_all_locator_widget(self) -> None:
        """Initialization of locator widgets to enable them if expected layer are available"""
        self._init_locator_widget(
            self.wdg_locator, [self.road_locator_filter, self.adress_locator_filter]
        )
        self._init_locator_widget(
            self.wdg_locator_intersection, [self.road_intersection_locator_filter]
        )

    def _init_locator_widget(
        self, locator_widget: QgsLocatorWidget, filters: List[VectorLayerLocatorFilter]
    ) -> None:
        """Initialization of locator widget to enable it if expected layer are available

        :param locator_widget: locator widget to initialize
        :type locator_widget: QgsLocatorWidget
        :param filters: list of filter use in locator
        :type filters: List[QgsLocatorFilter]
        """
        layer_names = []
        enabled = False

        # For all filter we check if the layer is available
        for filter in filters:
            layer_names.append(filter.get_search_layer_name())
            filter_enable = filter.can_be_used()
            # Disable filter
            filter.setEnabled(filter_enable)

            # Locator widget is enable if at least on filter is available
            enabled |= filter_enable
            if not filter_enable:
                locator_widget.invalidateResults()

        # Enable locator widget and define tooltip
        locator_widget.setEnabled(enabled)
        if not enabled:
            if len(layer_names) == 1:
                tooltip = self.tr("Couche '{}' non disponible").format(layer_names[0])
            else:
                tooltip = self.tr("Couches '{}' non disponibles").format(
                    ",".join(layer_names)
                )
        else:
            tooltip = ""
        locator_widget.setToolTip(tooltip)

    def road_selected(self, ids: List[int], geometry: QgsGeometry | None) -> None:
        """Enable intersection locator when road selected and define current id

        :param ids: selected roads id
        :type ids: List[int]
        :param geometry: selected roads geometry
        :type geometry: QgsGeometry
        """
        self.wdg_locator_intersection.invalidateResults()
        self.wdg_locator_intersection.setEnabled(geometry is not None)
        self.road_intersection_locator_filter.set_intersection_geometry(geometry)
        self.road_intersection_locator_filter.set_intersection_id(ids)

    def adress_selected(self):
        """Disable intersection locator when adress selected"""
        self.road_selected([], None)

    def _project_layer_updated(self) -> None:
        """Activate locator widget if layer are loaded"""
        self._init_all_locator_widget()
